/* -----------------------------------------------------------------------------*/
/*                              MQSeries Everyplace                             */
/*                                                                              */
/* Licensed Materials - Property of IBM                                         */
/*                                                                              */
/* 5639-I47                                                                     */
/*                                                                              */
/* (C) Copyright IBM Corp. 1999                                                 */
/* -----------------------------------------------------------------------------*/
/***********************************************************************
 *
 * Copyright (c) 1994-1999 3Com Corporation or its subsidiaries.
 * All rights reserved.
 *
 * PROJECT:  Pilot
 * FILE:     Starter.c
 * AUTHOR:   Roger Flores: May 20, 1997
 *
 * DECLARER: Starter
 *
 * DESCRIPTION:
 *	  
 *
 **********************************************************************/

#include <Pilot.h>
#include <SysEvtMgr.h>

/* The MQSeries Everyplace native C header file */
#include <hmq.h>
#include "MQeExample.h"

/***********************************************************************
 *
 *   Internal Structures
 *
 ***********************************************************************/
typedef struct 
{
   Byte replaceme;
} StarterPreferenceType;

typedef struct 
{
   Byte replaceme;
} StarterAppInfoType;

typedef StarterAppInfoType* StarterAppInfoPtr;



/***********************************************************************
 *
 *   Internal Constants
 *
 ***********************************************************************/
#define appFileCreator            'strt'
#define appVersionNum              0x01
#define appPrefID                  0x00
#define appPrefVersionNum          0x01


// Define the minimum OS version we support
#define ourMinVersion   sysMakeROMVersion(2,0,0,sysROMStageRelease,0)

/***********************************************************************
 *
 *   Internal Functions
 *
 ***********************************************************************/
void MainFormDoMQe( void );
FieldPtr SetFieldTextFromHandle(Word fieldID, Handle txtH);
FieldPtr SetFieldTextFromStr(Word fieldID, CharPtr strP);

/***********************************************************************
 *
 * FUNCTION:    RomVersionCompatible
 *
 * DESCRIPTION: This routine checks that a ROM version is meet your
 *              minimum requirement.
 *
 * PARAMETERS:  requiredVersion - minimum rom version required
 *                                (see sysFtrNumROMVersion in SystemMgr.h 
 *                                for format)
 *              launchFlags     - flags that indicate if the application 
 *                                UI is initialized.
 *
 * RETURNED:    error code or zero if rom is compatible
 *
 * REVISION HISTORY:
 *
 ***********************************************************************/
static Err RomVersionCompatible(DWord requiredVersion, Word launchFlags)
{
   DWord romVersion;

   // See if we're on in minimum required version of the ROM or later.
   FtrGet(sysFtrCreator, sysFtrNumROMVersion, &romVersion);
   if (romVersion < requiredVersion)
   {
      if ((launchFlags & (sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp)) ==
          (sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp))
      {
         FrmAlert (RomIncompatibleAlert);

         // Pilot 1.0 will continuously relaunch this app unless we switch to 
         // another safe one.
         if (romVersion < sysMakeROMVersion(2,0,0,sysROMStageRelease,0))
            AppLaunchWithCommand(sysFileCDefaultApp, sysAppLaunchCmdNormalLaunch, NULL);
      }

      return(sysErrRomIncompatible);
   }

   return(0);
}

/***********************************************************************
 *
 * FUNCTION:    GetObjectPtr
 *
 * DESCRIPTION: This routine returns a pointer to an object in the current
 *              form.
 *
 * PARAMETERS:  formId - id of the form to display
 *
 * RETURNED:    VoidPtr
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static VoidPtr GetObjectPtr(Word objectID)
{
   FormPtr frmP;

   frmP = FrmGetActiveForm();
   return(FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, objectID)));
}

/***********************************************************************
 *
 * FUNCTION:    MainFormInit
 *
 * DESCRIPTION: This routine initializes the MainForm form.
 *
 * PARAMETERS:  frm - pointer to the MainForm form.
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void MainFormInit(FormPtr frmP)
{
}

/***********************************************************************
 *
 * FUNCTION:    MainFormDoCommand
 *
 * DESCRIPTION: This routine performs the menu command specified.
 *
 * PARAMETERS:  command  - menu item id
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean MainFormDoCommand(Word command)
{
   Boolean handled = false;


   switch (command)
   {
      case MainOptionsAboutMQeExampleApp:
         MenuEraseStatus (0);
         AbtShowAbout (appFileCreator);
         handled = true;
         break;

   }
   return handled;
}

/***********************************************************************
 *
 * FUNCTION:    MainFormHandleEvent
 *
 * DESCRIPTION: This routine is the event handler for the 
 *              "MainForm" of this application.
 *
 * PARAMETERS:  eventP  - a pointer to an EventType structure
 *
 * RETURNED:    true if the event has handle and should not be passed
 *              to a higher level handler.
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean MainFormHandleEvent(EventPtr eventP)
{
   Boolean handled = false;
   FormPtr frmP;


   switch (eventP->eType)
   {
      case menuEvent:
         return MainFormDoCommand(eventP->data.menu.itemID);

      case frmOpenEvent:
         frmP = FrmGetActiveForm();
         MainFormInit( frmP);
         FrmDrawForm ( frmP);
         handled = true;
         break;

      case ctlSelectEvent:
         MainFormDoMQe();
         break;


      default:
         break;

   }

   return handled;
}

/***********************************************************************
 *
 * FUNCTION:    AppHandleEvent
 *
 * DESCRIPTION: This routine loads form resources and set the event
 *              handler for the form loaded.
 *
 * PARAMETERS:  event  - a pointer to an EventType structure
 *
 * RETURNED:    true if the event has handle and should not be passed
 *              to a higher level handler.
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean AppHandleEvent( EventPtr eventP)
{
   Word formId;
   FormPtr frmP;

   if (eventP->eType == frmLoadEvent)
   {
      // Load the form resource.
      formId = eventP->data.frmLoad.formID;
      frmP = FrmInitForm(formId);
      FrmSetActiveForm(frmP);

      // Set the event handler for the form.  The handler of the currently
      // active form is called by FrmHandleEvent each time is receives an
      // event.
      switch (formId)
      {
         case MainForm:
            FrmSetEventHandler(frmP, MainFormHandleEvent);
            break;

         default:
//              ErrFatalDisplay("Invalid Form Load Event");
            break;

      }
      return true;
   }

   return false;
}

/***********************************************************************
 *
 * FUNCTION:    AppEventLoop
 *
 * DESCRIPTION: This routine is the event loop for the application.  
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void AppEventLoop(void)
{
   Word error;
   EventType event;

   do
   {
      EvtGetEvent(&event, evtWaitForever);


      if (! SysHandleEvent(&event))
         if (! MenuHandleEvent(0, &event, &error))
            if (! AppHandleEvent(&event))
               FrmDispatchEvent(&event);

   } while (event.eType != appStopEvent);
}

/***********************************************************************
 *
 * FUNCTION:     MainFormDoMQe
 *
 * DESCRIPTION:  Simple MQe program. Initialise a queue manager session,
 *                    allocate a MQeMsgObject, put some data in the message,
 *                    put the message to a queue "SYSTEM.DEFAULT.LOCAL.QUEUE" on
 *                    a queue manager called "ExampleQM". Then do a get message 
 *                call to get the first message off this queue. Free the 
 *                    two message handles and terminate the session.         
 *                      
 *
 * PARAMETERS:   nothing
 *
 * RETURNED:     nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
void MainFormDoMQe( void )
{
   MQEHSESS   hSess;
   MQEHFIELDS hPutMsg;
   MQEHFIELDS hGetMsg;
   MQEINT32   compcode;
   MQEINT32   reason;
   MQEPMO     pmo = MQEPMO_DEFAULT; /* Set default put message options */
   MQEGMO     gmo = MQEGMO_DEFAULT; /* Set default get message options */
   char       errorcode[4];
   long       time;

   /* Set a message on the Palm screen */
   SetFieldTextFromStr( MainMessageField, "Initializing session" );

   /* Initialize the session: connect to the local queue manager   */
   hSess = MQeInitialize("MQeExample", &compcode, &reason);
   if (compcode != MQECC_OK)
   {
      /* Set an error message on the Palm screen */
      SetFieldTextFromStr( MainMessageField, "Error in MQeInitialize" );
      StrIToA( errorcode, reason );
      SetFieldTextFromStr( MainErrorField, errorcode );

      return;
   }

   /* Allocate memory for an MQeMsgObject object */
   hPutMsg = MQeFieldsAlloc( hSess, MQE_OBJECT_TYPE_MQE_MSGOBJECT, &compcode, &reason );
   if (compcode == MQECC_OK)
   {
      /* Put ASCII text "Hello World" into the MQeMsgObject in a field named "AsciiField" */
      MQeFieldsPut( hSess, hPutMsg, "AsciiField", MQE_TYPE_ASCII, "Hello World!", StrLen("Hello World!"), &compcode, &reason);
   }
   else
   {
      /* Set an error message on the Palm screen */
      SetFieldTextFromStr( MainMessageField, "Error in MQeFieldsAlloc" );
      StrIToA( errorcode, reason );
      SetFieldTextFromStr( MainErrorField, errorcode );

      return;
   }

   /* Set a message on the Palm screen */
   SetFieldTextFromStr( MainMessageField, "Putting a single message" );

   /* Put the MQeMsgObject to a Queue and Queue Manager (IP address, port, adapter and adapter command for the 
  queue manager "ExampleQM" need to be defined in the Preferences) */
   MQeQMgrPutMsg( hSess, "ExampleQM", "SYSTEM.DEFAULT.LOCAL.QUEUE", &pmo, hPutMsg, &compcode, &reason );
   if (compcode != MQECC_OK)
   {
      /* Set an error message on the Palm screen */
      SetFieldTextFromStr( MainMessageField, "Error in MQeQMgrPutMsg" );
      StrIToA( errorcode, reason );
      SetFieldTextFromStr( MainErrorField, errorcode );

      /* Wait for 3 seconds, so that users can see the error message */
      time = TimGetSeconds() + 3;
      while ( TimGetSeconds() < time ){}

   }

   /* Set a message on the Palm screen */
   SetFieldTextFromStr( MainMessageField, "Getting a single message" );

   /* Get the first message off the same Queue and Queue Manager */
   hGetMsg = MQeQMgrGetMsg( hSess, "ExampleQM", "SYSTEM.DEFAULT.LOCAL.QUEUE", &gmo, NULL, &compcode, &reason );
   if (compcode != MQECC_OK)
   {
      /* Set an error message on the Palm screen */
      SetFieldTextFromStr( MainMessageField, "Error in MQeQMgrGetMsg" );
      StrIToA( errorcode, reason );
      SetFieldTextFromStr( MainErrorField, errorcode );
            
      /* Wait for 3 seconds, so that users can see the error message */
      time = TimGetSeconds() + 3;
      while ( TimGetSeconds() < time ){}
   }


   /* Free the memory held by the hPutMsg handle */
   if (hPutMsg != MQEHANDLE_NULL)
   {
      MQeFieldsFree( hSess, hPutMsg, &compcode, &reason );
      if (compcode != MQECC_OK)
      {
         /* Set an error message on the Palm screen */
         SetFieldTextFromStr( MainMessageField, "Error in MQeFieldsFree" );
         StrIToA( errorcode, reason );
         SetFieldTextFromStr( MainErrorField, errorcode );

         /* Wait for 3 seconds, so that users can see the error message */
         time = TimGetSeconds() + 3;
         while ( TimGetSeconds() < time ){}
      }
   }

   /* Free the memory held by the hGetMsg handle */
   if (hGetMsg != MQEHANDLE_NULL)
   {
      MQeFieldsFree( hSess, hGetMsg, &compcode, &reason );
      if (compcode != MQECC_OK)
      {

         /* Set an error message on the Palm screen */
         SetFieldTextFromStr( MainMessageField, "Error in MQeFieldsFree" );
         StrIToA( errorcode, reason );
         SetFieldTextFromStr( MainErrorField, errorcode );

         /* Wait for 3 seconds, so that users can see the error message */
         time = TimGetSeconds() + 3;
         while ( TimGetSeconds() < time ){}
      }
   }

   /* Set a message on the Palm screen */
   SetFieldTextFromStr( MainMessageField, "Terminating the session" );

   /* Terminate the session */
   MQeTerminate( hSess, &compcode, &reason );
   if (compcode != MQECC_OK)
   {
      /* Set an error message on the Palm screen */
      SetFieldTextFromStr( MainMessageField, "Error in MQeTerminate" );
      StrIToA( errorcode, reason );
      SetFieldTextFromStr( MainErrorField, errorcode );
      return;
   }

   /* Set a message on the Palm screen */
   SetFieldTextFromStr( MainMessageField, "MQe code executed!" );

}

/***********************************************************************
 *
 * FUNCTION:    SetFieldTextFromHandle
 *
 * DESCRIPTION: This routine simplifies the setting of hext in a field.
 *
 * PARAMETERS:  fieldID - ID of field to populate
 *              txtH - Handle to memory containing string to use.
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
FieldPtr SetFieldTextFromHandle(Word fieldID, Handle txtH)
{
   Handle      oldTxtH;
   FormPtr     frm = FrmGetActiveForm();
   FieldPtr    fldP;

   // get the field and the field's current text handle.
   fldP     = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, fieldID));
   ErrNonFatalDisplayIf(!fldP, "missing field");
   oldTxtH  = FldGetTextHandle(fldP);

   // Set the field's text to the new text.
   FldSetTextHandle(fldP, txtH);
   FldDrawField(fldP);

   // Free the handle AFTER we call FldSetTextHandle().
   if (oldTxtH)
      MemHandleFree(oldTxtH);

   return fldP;
}  

/***********************************************************************
 *
 * FUNCTION:    SetFieldTextFromStr
 *
 * DESCRIPTION: This routine simplifies the setting of hext in a field.
 *
 * PARAMETERS:  fieldID - ID of field to populate
 *              strP - Pointer to string to use.
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
FieldPtr SetFieldTextFromStr(Word fieldID, CharPtr strP)
{
   Handle      txtH;

   // get some space in which to stash the string.
   txtH  = MemHandleNew(StrLen(strP) + 1);
   if (!txtH)
      return NULL;

   // copy the string to the locked handle.
   StrCopy(MemHandleLock(txtH), strP);

   // unlock the string handle.
   MemHandleUnlock(txtH);

   // set the field to the handle
   return SetFieldTextFromHandle(fieldID, txtH);
} 

/***********************************************************************
 *
 * FUNCTION:     AppStart
 *
 * DESCRIPTION:  Do the quick test.
 *
 * PARAMETERS:   nothing
 *
 * RETURNED:     Err value 0 if nothing went wrong
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Err AppStart(void)
{
   return 0;
}

/***********************************************************************
 *
 * FUNCTION:    AppStop
 *
 * DESCRIPTION: Save the current state of the application.
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void AppStop(void)
{
   StarterPreferenceType prefs;

   // Write the saved preferences / saved-state information.  This data 
   // will be backed up during a HotSync.
   PrefSetAppPreferences (appFileCreator, appPrefID, appPrefVersionNum, 
                          &prefs, sizeof (prefs), true);
}

/***********************************************************************
 *
 * FUNCTION:    StarterPilotMain
 *
 * DESCRIPTION: This is the main entry point for the application.
 * PARAMETERS:  cmd - word value specifying the launch code. 
 *              cmdPB - pointer to a structure that is associated with the launch code. 
 *              launchFlags -  word value providing extra information about the launch.
 *
 * RETURNED:    Result of launch
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static DWord StarterPilotMain(Word cmd, Ptr cmdPBP, Word launchFlags)
{
   Err error;


   error = RomVersionCompatible (ourMinVersion, launchFlags);
   if (error) return(error);


   switch (cmd)
   {
      case sysAppLaunchCmdNormalLaunch:
         error = AppStart();
         if (error)
            return error;

         FrmGotoForm(MainForm);
         AppEventLoop();
         AppStop();
         break;

      default:
         break;
   }

   return 0;
}

/***********************************************************************
 *
 * FUNCTION:    PilotMain
 *
 * DESCRIPTION: This is the main entry point for the application.
 *
 * PARAMETERS:  cmd - word value specifying the launch code. 
 *              cmdPB - pointer to a structure that is associated with the launch code. 
 *              launchFlags -  word value providing extra information about the launch.
 * RETURNED:    Result of launch
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
DWord PilotMain( Word cmd, Ptr cmdPBP, Word launchFlags)
{
   return StarterPilotMain(cmd, cmdPBP, launchFlags);
}
