/* -----------------------------------------------------------------------------*/
/*                              MQSeries Everyplace                             */
/*                                                                              */
/* Licensed Materials - Property of IBM                                         */
/*                                                                              */
/* 5639-I47                                                                     */
/*                                                                              */
/* (C) Copyright IBM Corp. 1999                                                 */
/* -----------------------------------------------------------------------------*/
/** @@HEADER_H_INC */

#ifndef HMQ_H_INCLUDED
   #define HMQ_H_INCLUDED 1

/* 64 bit structures defined in terms of platform's 32 bit base 
   types. These structs (not typedefs) are part of the platform
   independant layer.
   These are for allocating space, arithmetic operations are not
   supported.
 */

#if defined(__cplusplus)
extern "C" {
#endif

#define MQE_OS_PALMOS   1
#define MQE_OS_WINCE    2
#define MQE_OS_LINUX    3
#define MQE_OS_EPOC     4
#define MQE_OS_ZAURUSOS 5

#ifndef MQE_OS
   /* default MQE_OS is always MQE_OS_PALMOS */
#define MQE_OS MQE_OS_PALMOS
#endif

/* PalmOS specific defines */
#if MQE_OS == MQE_OS_PALMOS
typedef unsigned char  MQEBYTE;
typedef char           MQECHAR;
typedef short          MQEINT16;
typedef long           MQEINT32;
typedef void           MQEVOID;
typedef MQECHAR *      PMQECHAR;
typedef MQEVOID *      PMQEVOID;
typedef MQEINT32       MQEHANDLE;
typedef float          MQEFLOAT;
typedef double         MQEDOUBLE;

typedef struct MQEINT64
{
   MQEINT32        hi;
   unsigned long   lo;
} MQEINT64;

#define MQECONST        const
#define MQETCHAR        MQECHAR
#define MQTS(_s) (_s)
#define MQEENTRY

extern MQEENTRY MQEINT32 MQE_SIZEOF(MQEBYTE);

/* EPOC specific defines */
#elif MQE_OS == MQE_OS_EPOC

typedef unsigned char  MQEBYTE;
typedef char           MQECHAR;
typedef short          MQEINT16;
typedef long           MQEINT32;
typedef void           MQEVOID;
typedef MQECHAR *      PMQECHAR;
typedef MQEVOID *      PMQEVOID;
typedef MQEINT32       MQEHANDLE;
typedef float          MQEFLOAT;
typedef double         MQEDOUBLE;

typedef struct MQEINT64
{
   MQEINT32          hi;
   unsigned long     lo;
} MQEINT64;

#define MQECONST        const
#define MQETCHAR       MQEBYTE
#define MQTS(_s) (_s)
#define MQEENTRY __declspec(dllimport)

#define MQE_SIZEOF mqe_sizeof
extern MQEENTRY MQEINT32 mqe_sizeof(MQEBYTE);


/* WINCE specific defines */
#elif MQE_OS == MQE_OS_WINCE

typedef unsigned char  MQEBYTE;
typedef char           MQECHAR;
typedef short          MQEINT16;
typedef long           MQEINT32;
typedef void           MQEVOID;
typedef MQECHAR *      PMQECHAR;
typedef MQEVOID *      PMQEVOID;
typedef MQEINT32       MQEHANDLE;
typedef float          MQEFLOAT;
typedef double         MQEDOUBLE;

typedef struct MQEINT64
{
   MQEINT32          hi;
   unsigned long     lo;
} MQEINT64;

#define MQECONST        const
#define MQETCHAR       MQEBYTE
#define MQTS(_s) (_s)
#define MQEENTRY __declspec(dllimport)

#define MQE_SIZEOF mqe_sizeof
   extern MQEENTRY MQEINT32 mqe_sizeof(MQEBYTE);

/* Linux specific defines */
#elif MQE_OS == MQE_OS_LINUX

typedef unsigned char  MQEBYTE;
typedef char           MQECHAR;
typedef short          MQEINT16;
typedef long           MQEINT32;
typedef void           MQEVOID;
typedef MQECHAR *      PMQECHAR;
typedef MQEVOID *      PMQEVOID;
typedef MQEINT32       MQEHANDLE;
typedef float          MQEFLOAT;
typedef double         MQEDOUBLE;

typedef struct MQEINT64
{
   MQEINT32            hi;
   unsigned long       lo;
} MQEINT64;

#define MQECONST        const
#define MQETCHAR       MQEBYTE
#define MQTS(_s) (_s)
#define MQEENTRY 

#define MQE_SIZEOF mqe_sizeof
extern MQEENTRY MQEINT32 mqe_sizeof(MQEBYTE);

#else 
#error "MQE_OS defined to unknown OS"
#endif /* MQE_OS == ... */


#define MQEHFIELDS     MQEHANDLE
#define MQEHSESS       MQEHANDLE
#define MQEHATTRB      MQEHANDLE
#define MQEHANDLE_NULL ((MQEHANDLE)0)

#define MQE_TYPE_UNTYPED                         0xC0
#define MQE_TYPE_ASCII                           0xC1
#define MQE_TYPE_UNICODE                         0xC2
#define MQE_TYPE_BOOLEAN                         0xC3
#define MQE_TYPE_BYTE                            0xC4
#define MQE_TYPE_SHORT                           0xC5
#define MQE_TYPE_INT16                           0xC5
#define MQE_TYPE_INT                             0xC6
#define MQE_TYPE_INT32                           0xC6
#define MQE_TYPE_LONG                            0xC7       
#define MQE_TYPE_INT64                           0xC7       
#define MQE_TYPE_FLOAT                           0xC8       
#define MQE_TYPE_DOUBLE                          0xC9
#define MQE_TYPE_ARRAYELEMENTS                   0xCA
#define MQE_TYPE_FIELDS                          0xCB
#define MQE_TYPE_RESERVE_C                       0xCC
#define MQE_TYPE_RESERVE_D                       0xCD
#define MQE_TYPE_RESERVE_E                       0xCE
#define MQE_TYPE_RESERVE_F                       0xCF


/* Completion code defines */
#define MQECC_OK                                     0
#define MQECC_WARNING                               -1
#define MQECC_ERROR                                 -2


/* Exception index numbers                                                     */
#define   MQE_EXCEPT_UNCODED                         0
#define   MQE_EXCEPT_DEBUG                           1
#define   MQE_EXCEPT_NOT_SUPPORTED                   2
#define   MQE_EXCEPT_SYNTAX                          3
#define   MQE_EXCEPT_TYPE                            4
#define   MQE_EXCEPT_COMMAND                         5
#define   MQE_EXCEPT_NOT_FOUND                       6
#define   MQE_EXCEPT_DATA                            7
#define   MQE_EXCEPT_BAD_REQUEST                     8
#define   MQE_EXCEPT_STOPPED                         9
#define   MQE_EXCEPT_CLOSED                         10
#define   MQE_EXCEPT_DUPLICATE                      11
#define   MQE_EXCEPT_NOT_ALLOWED                    12
#define   MQE_EXCEPT_RULE                           13
#define   MQE_EXCEPT_TIMEOUT                        14

/*                                                                             */
#define   MQE_EXCEPT_CHNL_ATTRIBUTES                20
#define   MQE_EXCEPT_CHNL_DESTINATION               22
#define   MQE_EXCEPT_CHNL_LIMIT                     23
#define   MQE_EXCEPT_CHNL_ID                        24
#define   MQE_EXCEPT_CHNL_OVERRUN                   25
#define   MQE_EXCEPT_CHNL_OPEN                      28

/*                                                                             */
#define   MQE_EXCEPT_TRNSPORT_QMGR                  40
#define   MQE_EXCEPT_TRNSPORT_REQUEST               41

/*                                                                             */
#define   MQE_EXCEPT_QMGR_NOT_ACTIVE               100
#define   MQE_EXCEPT_QMGR_INVALID_QMGR_NAME        101
#define   MQE_EXCEPT_QMGR_ACTIVATED                102
#define   MQE_EXCEPT_QMGR_ALREADY_EXISTS           103
#define   MQE_EXCEPT_QMGR_INVALID_Q_NAME           104
#define   MQE_EXCEPT_QMGR_Q_EXISTS                 105
#define   MQE_EXCEPT_QMGR_UNKNOWN_QMGR             106
#define   MQE_EXCEPT_QMGR_Q_NOT_EMPTY              107
#define   MQE_EXCEPT_QMGR_Q_DOES_NOT_EXIST         108
#define   MQE_EXCEPT_QMGR_Q_IN_USE                 109
#define   MQE_EXCEPT_QMGR_WRONG_QTYPE              110
#define   MQE_EXCEPT_QMGR_INVALID_CHANNEL          111
#define   MQE_EXCEPT_QMGR_SECURE_MSG_DECODE_FAILED 112

/*                                                                            */
#define   MQE_EXCEPT_Q_NO_MSG_AVAILABLE            120
#define   MQE_EXCEPT_Q_NO_MATCHING_MSG             121
#define   MQE_EXCEPT_Q_NO_MATCHING_MSG_LISTENER    122
#define   MQE_EXCEPT_Q_INVALID_PRIORITY            124
#define   MQE_EXCEPT_Q_FULL                        125
#define   MQE_EXCEPT_Q_MSG_TOO_LARGE               126
#define   MQE_EXCEPT_Q_NOT_ACTIVE                  127
#define   MQE_EXCEPT_Q_ACTIVE                      128
#define   MQE_EXCEPT_Q_INVALID_NAME                129

/*                                                                            */
#define   MQE_EXCEPT_RAS_DIAL_FAILED                 201
#define   MQE_EXCEPT_RAS_GET_PROJECTION_INFO_FAILED  202
#define   MQE_EXCEPT_RAS_HANGUP_FAILED               203

/*                                                                            */
#define   MQE_EXCEPT_CONNECT_ADAPTER_NOT_ACTIVE      210
#define   MQE_EXCEPT_CONNECT_INVALID_DEFINITION      211

/*                                                                            */
#define   MQE_EXCEPT_REG_NOT_FOUND                   301
#define   MQE_EXCEPT_REG_NULL_NAME                   302
#define   MQE_EXCEPT_REG_ALREADY_EXISTS              303
#define   MQE_EXCEPT_REG_DOES_NOT_EXIST              304
#define   MQE_EXCEPT_REG_NOT_ACTIVATED               305
#define   MQE_EXCEPT_REG_OPEN_FAILED                 306
#define   MQE_EXCEPT_REG_INVALID_SESSION             307
#define   MQE_EXCEPT_REG_NOT_DEFINED                 308
#define   MQE_EXCEPT_REG_INVALID_NAME                309
#define   MQE_EXCEPT_REG_LOWER_CASE                  310
#define   MQE_EXCEPT_REG_ADD_FAILED                  311
#define   MQE_EXCEPT_REG_DELETE_FAILED               312
#define   MQE_EXCEPT_REG_READ_FAILED                 313
#define   MQE_EXCEPT_REG_UPDATE_FAILED               314
#define   MQE_EXCEPT_REG_LIST_FAILED                 315
#define   MQE_EXCEPT_REG_SEARCH_FAILED               316

/*                                                                            */
#define   MQE_EXCEPT_PRIVATE_REG_BAD_PIN             350
#define   MQE_EXCEPT_PRIVATE_REG_ACTIVATE_FAILED     351
#define   MQE_EXCEPT_PRIVATE_REG_NOT_OPEN            352

/*                                                                            */
#define   MQE_EXCEPT_MINI_CERTREG_BAD_PIN            360
#define   MQE_EXCEPT_MINI_CERTREG_ACTIVATE_FAILED    361
#define   MQE_EXCEPT_MINI_CERTREG_NOT_OPEN           362

/*                                                                            */
#define   MQE_EXCEPT_PUBLIC_REG_ACTIVATE_FAILED      370
#define   MQE_EXCEPT_PUBLIC_REG_INVALID_REQUEST      371

/*                                                                            */
#define   MQE_EXCEPT_ADMIN_NOT_ADMIN_MSG             400

/*                                                                            */
#define   MQE_EXCEPT_AUTHENTICATE                    500
#define   MQE_EXCEPT_S_CIPHER                        501
#define   MQE_EXCEPT_S_INVALID_SIGNATURE             502
#define   MQE_EXCEPT_S_CERTIFICATE_EXPIRED           503
#define   MQE_EXCEPT_S_INVALID_ATTRIBUTE             504
#define   MQE_EXCEPT_S_MINICERT_NOT_AVAILABLE        505
#define   MQE_EXCEPT_S_REGISTRY_NOT_AVAILABLE        506
#define   MQE_EXCEPT_S_BAD_INTEGRITY                 507
#define   MQE_EXCEPT_S_NO_PRESET_KEY_AVAILABLE       508
#define   MQE_EXCEPT_S_MISSING_SECTION               509

#define   MQE_EXCEPT_BUFFER_OVERFLOW                  15
#define   MQE_EXCEPT_INVALID_HANDLE                   16
#define   MQE_EXCEPT_INVALID_ARGUMENT                 17
#define   MQE_EXCEPT_ALLOCATION_FAILED                18
#define   MQE_EXCEPT_FAILURE                          19

#define   MQE_EXCEPT_NETWORK_ERROR                   600
#define   MQE_EXCEPT_NETWORK_ERROR_OPEN              601
#define   MQE_EXCEPT_NETWORK_ERROR_READ              602
#define   MQE_EXCEPT_NETWORK_ERROR_WRITE             603

#define   MQE_EXCEPT_EOF                             700 
#define   MQE_EXCEPT_NON_MQE_SYSTEM_EXCEPTION        701 
#define   MQE_EXCEPT_TRUNCATED                       702 

#define   MQE_EXCEPT_PLATFORM_LIB_LOAD_FAILED       2000
#define   MQE_EXCEPT_PLATFORM_LIB_STILL_OPEN        2001
#define   MQE_WARN_PLATFORM_LIB_ALREADYOPEN         2500
#define   MQE_WARN_SESSION_DELETED                  2501
#define   MQE_WARN_FIELD_DATA_TRUNCATED             2502


#define   MQE_EXCEPT_ADMIN_UNKNOWN_CHARACTERISTIC   3001
#define   MQE_EXCEPT_ADMIN_UNDEFINED_ACTION         3002

#define   MQE_EXCEPT_ADAPTER_HTTP_ERROR             4000

/* Max length of well known parameters */
#define MQE_MAX_QMGR_NAME_LENGTH                128
#define MQE_MAX_Q_NAME_LENGTH                   128
#define MQE_MAX_SESSION_NAME                     20

/* MQeFields OPTIONS */
#define MQE_FIELDS_OPTION_NONE                    0
#define MQE_FIELDS_OPTION_ALL_FIELDS              1
#define MQE_FIELDS_OPTION_REPLACE                 2

/* MQeQMgr OPTIONS */
#define MQE_QMGR_OPTION_NONE              0x00000000L
#define MQE_QMGR_OPTION_PUT_DEFAULT       0x00000000L
#define MQE_QMGR_OPTION_PUT_MASK          0x0000000fL
#define MQE_QMGR_OPTION_PUT_ASYNCHRONOUS  0x00000001L
#define MQE_QMGR_OPTION_PUT_SYNCHRONOUS   0x00000002L
#define MQE_QMGR_OPTION_BROWSE_LOCK       0x00000010L 
#define MQE_QMGR_OPTION_BROWSE_JUST_UID   0x00000020L 
#define MQE_QMGR_OPTION_CONFIRMID         0x00000100L
#define MQE_QMGR_OPTION_CONFIRM_GETMSG    0x00000300L
#define MQE_QMGR_OPTION_CONFIRM_PUTMSG    0x00000500L

/* Trace options */
#define MQE_TRACE_CMD_START                        1
#define MQE_TRACE_CMD_STOP                         2
#define MQE_TRACE_CMD_SET_MASK                     3
#define MQE_TRACE_CMD_SET_HANDLER                  4
#define MQE_TRACE_OPTION_APP_MSG              0x0001
#define MQE_TRACE_OPTION_APP_INFO             0x0002
#define MQE_TRACE_OPTION_APP_WARNING          0x0004
#define MQE_TRACE_OPTION_APP_ERROR            0x0008
#define MQE_TRACE_OPTION_APP_DEBUG            0x0010
#define MQE_TRACE_OPTION_APP_ALL              0x001f
#define MQE_TRACE_OPTION_SYS_MSG              0x0100
#define MQE_TRACE_OPTION_SYS_INFO             0x0200
#define MQE_TRACE_OPTION_SYS_WARNING          0x0400
#define MQE_TRACE_OPTION_SYS_ERROR            0x0800
#define MQE_TRACE_OPTION_SYS_DEBUG            0x1000
#define MQE_TRACE_OPTION_SYS_ALL              0x1f00

/* MQe fields object type */
#define MQE_OBJECT_TYPE_MQE_FIELDS                  "com.ibm.mqe.MQeFields"
#define MQE_OBJECT_TYPE_MQE_MSGOBJECT               "com.ibm.mqe.MQeMsgObject"
#define MQE_OBJECT_TYPE_MQE_ADMIN_MSG               "com.ibm.mqe.MQeAdminMsg"
#define MQE_OBJECT_TYPE_MQE_QUEUE_ADMIN_MSG         "com.ibm.mqe.administration.MQeQueueAdminMsg"
#define MQE_OBJECT_TYPE_MQE_REMOTE_QUEUE_ADMIN_MSG  "com.ibm.mqe.administration.MQeRemoteQueueAdminMsg"
#define MQE_OBJECT_TYPE_MQE_CONNECTION_ADMIN_MSG    "com.ibm.mqe.administration.MQeConnectionAdminMsg"
#define MQE_OBJECT_TYPE_MQE_QUEUE_MANAGER_ADMIN_MSG "com.ibm.mqe.administration.MQeQueueManagerAdminMsg"
#define MQE_OBJECT_TYPE_MQE_FRAGMENTOR              "com.ibm.mqe.MQeFragmentor"
#define MQE_OBJECT_TYPE_MQE_FIELDS_UNTYPE           ""

/* MsgObject field name */
#define MQE_MSG_CORRELID              "\260"     /* 0xB0 - Msg_CorrelID     */
#define MQE_MSG_MSGID                 "\261"     /* 0xB1 - Msg_MsgID        */
#define MQE_MSG_ORIGIN_QMGR           "\262"     /* 0xB2 - Msg_OriginQMgr   */
#define MQE_MSG_PRIORITY              "\263"     /* 0xB3 - Msg_Priority     */
#define MQE_MSG_TIME                  "\264"     /* 0xB4 - Msg_Time         */
#define MQE_MSG_REPLYTO_Q             "\265"     /* 0xB5 - Msg_ReplyToQ     */
#define MQE_MSG_REPLYTO_QMGR          "\266"     /* 0xB6 - Msg_ReplyToQMgr  */
#define MQE_MSG_STYLE                 "\267"     /* 0xB7 - Msg_Style        */
#define MQE_MSG_LOCKID                "\270"     /* 0xB8 - Msg_LockID       */
#define MQE_MSG_RESEND                "\271"     /* 0xB9 - Msg_Resend       */
#define MQE_MSG_EXPIRETIME            "\272"     /* 0xBA - Msg_ExpireTime   */
#define MQE_MSG_WRAPMSG               "\273"     /* 0xBB - Msg_WrapMsg      */

/*********************************************************************/
/*  Values Related to MQEBMO Structure                               */
/*********************************************************************/

/* Structure Identifier */
#define MQEBMO_STRUC_ID "BRWM"

/* Structure Identifier (array form) */
#define MQEBMO_STRUC_ID_ARRAY 'B','R','W','M'

/* Structure Version Number */
#define MQEBMO_VERSION_1       1L
#define MQEBMO_CURRENT_VERSION 1L

/*********************************************************************/
/*  Values Related to MQEGMO Structure                               */
/*********************************************************************/

/* Structure Identifier */
#define MQEGMO_STRUC_ID "GETM"

/* Structure Identifier (array form) */
#define MQEGMO_STRUC_ID_ARRAY 'G','E','T','M'

/* Structure Version Number */
#define MQEGMO_VERSION_1       1L
#define MQEGMO_CURRENT_VERSION 1L

/*********************************************************************/
/*  Values Related to MQEPMO Structure                               */
/*********************************************************************/

/* Structure Identifier */
#define MQEPMO_STRUC_ID "PUTM"

/* Structure Identifier (array form) */
#define MQEPMO_STRUC_ID_ARRAY 'P','U','T','M'

/* Structure Version Number */
#define MQEPMO_VERSION_1       1L
#define MQEPMO_CURRENT_VERSION 1L

/* MQeBrowseMsg() Option data structure */
typedef struct tagMQeBrowseMsgOpts
{
   MQECHAR   StrucId[4];          /* Input  */
   MQEINT32   Version;            /* Input  */
   MQEINT32   Options;            /* Input  */
   MQEINT64 ConfirmId;            /* Input  */
   MQEHATTRB hAttrb;              /* Input  */
   MQEINT64 LockId;               /* Output */
   MQEINT64 Cookie;               /* Output */
} MQEBMO;

#define MQEBMO_DEFAULT  { { MQEBMO_STRUC_ID_ARRAY}, \
                          MQEBMO_VERSION_1,         \
                          MQE_QMGR_OPTION_NONE,     \
                          {0,},                     \
                          MQEHANDLE_NULL,           \
                          {0,},                     \
                          {0,} }

/* MQeGetMsg() Option data structure */
typedef struct tagMQeGetMsgOpts
{
   MQECHAR   StrucId[4];          /* Input  */
   MQEINT32   Version;            /* Input  */
   MQEINT32   Options;            /* Input  */
   MQEINT64 ConfirmId;            /* Input  */
   MQEHATTRB hAttrb;              /* Input  */
} MQEGMO;

#define MQEGMO_DEFAULT  { { MQEGMO_STRUC_ID_ARRAY}, \
                          MQEGMO_VERSION_1,         \
                          MQE_QMGR_OPTION_NONE,     \
                          {0,},                     \
                          MQEHANDLE_NULL }

/* MQePutMsg() Option data structure */
typedef struct tagMQePutMsgOpts
{
   MQECHAR   StrucId[4];          /* Input  */
   MQEINT32   Version;            /* Input  */
   MQEINT32   Options;            /* Input  */
   MQEINT64 ConfirmId;            /* Input  */
   MQEHATTRB hAttrb;              /* Input  */
} MQEPMO;

#define MQEPMO_DEFAULT  { { MQEPMO_STRUC_ID_ARRAY}, \
                          MQEPMO_VERSION_1,         \
                          MQE_QMGR_OPTION_NONE,     \
                          {0,},                     \
                          MQEHANDLE_NULL }

/* MQeField data structure */
typedef struct MQeField_st
{
   MQEBYTE    fd_datatype;
   MQEBYTE    __pad;
   MQEINT16   fd_namelen;                       /* field name < 32KB */
   MQECHAR  * fd_name;
   MQEVOID  * fd_data;
   MQEINT32   fd_datalen;
   MQEVOID  * fd_base;
} MQeField_t, MQEFIELD;

/* MQeFieldsPutByStructure() structure descriptor data structure */
typedef struct MQeFieldStructDescriptor_st
{
   PMQECHAR   sd_name;         /* Field name */
   MQEINT32    sd_namelen;     /* Length of field name */
   MQEBYTE    sd_datatype;     /* Type of field */
   MQEBYTE    sd_flags;        /* flags describing field layout in struct */
   MQEINT32    sd_dataoff;     /* data offset in struct */
   MQEINT32    sd_datalen;     /* (offset of) data length for field */
}  MQEFIELDDESC;

#define MQSTRUCT_LEN     0x1  /* struct offset sd_datalen is pointer to length, not number of elements */
#define MQSTRUCT_DATA    0x2  /* struct offset sd_dataoff is pointer to data, not start of data block */
#define MQSTRUCT_NODATA  0x4  /* ignore sd_dataoff */

#if 0
extern MQEENTRY const MQECHAR * MQeFields_Type;
extern MQEENTRY const MQECHAR * MQeMsgObject_Type;
extern MQEENTRY const MQECHAR * MQeFields_Untyped;
extern MQEENTRY const MQEINT16 mqe_BytesPerType[];
#endif

/**
  MQe System API
  */
extern MQEENTRY 
MQEHSESS  MQeInitialize( MQECHAR * pSessionName, MQEINT32 * pCompCode, MQEINT32 * pReason);

extern MQEENTRY 
MQEVOID   MQeTerminate ( MQEHSESS hSess, MQEINT32 * pCompCode, MQEINT32 * pReason);

extern MQEENTRY 
MQEINT32 MQeGetVersion  ( MQEINT32 * pCompCode, MQEINT32 * pReason);

extern MQEENTRY
MQEVOID MQeConfigCreateQMgr( MQECHAR * pQMgr, MQEINT32 * pCompCode, MQEINT32 * pReason);

extern MQEENTRY
MQEVOID MQeConfigDeleteQMgr( MQECHAR * pQMgr, MQEINT32 * pCompCode, MQEINT32 * pReason);
/**
     MQeTrace API
 */
extern MQEENTRY 
MQEVOID MQeTraceCmd ( MQEHSESS hSess, MQEINT32 cmd, MQEINT32 parm, MQEINT32 * pCompCode, MQEINT32 * pReason);        

extern MQEENTRY 
MQEVOID  MQeTrace ( MQEHSESS hSess, MQETCHAR * pTStr);

/**
  Queue Manager API
  */
extern MQEENTRY 
MQEVOID MQeQMgrAddMsgListener( MQEHSESS hSess, MQEVOID * pMsgListener, MQECHAR * pQueueName,
                               MQEHFIELDS hFlds, MQEINT32 * pCompCode, MQEINT32 * pReason) ;
extern MQEENTRY 
MQEINT32 MQeQMgrBrowseMsgs    ( MQEHSESS hSess, MQECHAR * pQMName, MQECHAR * pQName, MQEVOID * pBrowseMsgOpts, 
                                MQEHFIELDS hFilter, MQEHFIELDS pMsgs[], MQEINT32  nMsgs, 
                                MQEINT32 * pCompCode, MQEINT32 * pReason);
extern MQEENTRY 
MQEVOID  MQeQMgrConfirmMsg   ( MQEHSESS hSess, MQECHAR * pQMName, MQECHAR * pQName, MQEINT32 Options , 
                               MQEHFIELDS hMsg , MQEINT32 * pCompCode, MQEINT32 * pReason); 
extern MQEENTRY 
MQEINT32 MQeQMgrDeleteMsgs    ( MQEHSESS hSess, MQECHAR * pQMName, MQECHAR * pQName, 
                                MQEHFIELDS  pMsgs[], MQEINT32 nMsgs , MQEINT32 * pCompCode, MQEINT32 * pReason); 
extern MQEENTRY 
MQEHFIELDS MQeQMgrGetMsg       ( MQEHSESS hSess, MQECHAR * pQMName, MQECHAR * pQName, MQEVOID * pGetMsgOpts,
                                 MQEHFIELDS hFilter, MQEINT32 * pCompCode, MQEINT32 * pReason);
extern MQEENTRY 
MQEINT32 MQeQMgrGetName       ( MQEHSESS hSess, MQECHAR * pQMgrName, MQEINT32 qmNameLen, MQEINT32 * pCompCode, 
                                MQEINT32 * pReason); 
extern MQEENTRY 
MQEINT32 MQeQMgrGetQueueList  ( MQEHSESS hSess, MQEINT32 queueNameLen, MQECHAR * pQueueName,
                                MQEINT32 * pToken, MQEINT32 * pCompCode, MQEINT32 * pReason);
extern MQEENTRY 
MQEVOID MQeQMgrPutMsg        ( MQEHSESS hSess, MQECHAR * pQMName, MQECHAR * pQName, MQEVOID * pPutMsgOpts,
                               MQEHFIELDS hMsgObj, MQEINT32 * pCompCode, MQEINT32 * pReason); 
extern MQEENTRY
MQEVOID MQeQMgrRemoveMsgListener( MQEHSESS hSess, MQEVOID * pMsgListener, MQECHAR *
                                  pQueueName, MQEINT32 * pCompCode, MQEINT32 * pReason);
extern MQEENTRY 
MQEVOID MQeQMgrUndo         ( MQEHSESS hSess, MQECHAR * pQMName, MQECHAR * pQName, MQEINT64 * pConfirmId, 
                              MQEINT32 * pCompCode, MQEINT32 * pReason); 

extern MQEENTRY 
MQEINT32 MQeQMgrUnlockMsgs    ( MQEHSESS hSess, MQECHAR * pQMName, MQECHAR * pQName, MQEINT64 * pLockId, 
                                MQEHFIELDS  pMsgs[], MQEINT32  nMsgs, MQEINT32 * pCompCode, MQEINT32 * pReason); 
extern MQEENTRY 
MQEHFIELDS MQeQMgrWaitForMsg   ( MQEHSESS hSess, MQECHAR * pQMgrName, MQECHAR * pQName, MQEINT32 nMilliSeconds, 
                                 MQEHFIELDS hFilter, MQEINT32 * pCompCode, MQEINT32 * pReason); 
/**
     MQeFields Core APIs
 */

extern  MQEENTRY 
MQEHFIELDS MQeFieldsAlloc ( MQEHSESS hSess, MQECONST MQECHAR * Type, MQEINT32 * pCompCode, MQEINT32 * pReason);
extern  MQEENTRY 
MQEINT32  MQeFieldsDelete( MQEHSESS hSess, MQEHFIELDS hFlds, MQECHAR * pName, MQEINT32 * pCompCode, MQEINT32 * pReason);
extern  MQEENTRY 
MQEINT32  MQeFieldsDump  ( MQEHSESS hSess, MQEHFIELDS hFlds, MQEINT32 SrcOff, MQEBYTE pBuf[], MQEINT32 BufLen, 
                           MQEVOID * pBase, MQEINT32 * pCompCode, MQEINT32 * pReason);
extern  MQEENTRY 
MQEINT32  MQeFieldsDumpLength( MQEHSESS hSess, MQEHFIELDS hFlds, MQEINT32 * pCompCode, MQEINT32 * pReason);
extern  MQEENTRY 
MQEINT32  MQeFieldsEquals( MQEHSESS hSess, MQEHFIELDS hFlds1, MQEHFIELDS hFlds2, MQEINT32 * pCompCode, MQEINT32 * pReason);
extern  MQEENTRY 
MQEINT32  MQeFieldsFields( MQEHSESS hSess, MQEHFIELDS hFlds, MQEINT32 * pCompCode, MQEINT32 * pReason);
extern  MQEENTRY 
MQEVOID  MQeFieldsFree  ( MQEHSESS hSess, MQEHFIELDS hFlds, MQEINT32 * pCompCode, MQEINT32 * pReason);
extern  MQEENTRY 
MQEINT32  MQeFieldsGet   ( MQEHSESS hSess, MQEHFIELDS hFlds, MQECHAR *pName, MQEBYTE *pDataType, MQEVOID *pData, MQEINT32 nElements, MQEVOID *pBase, MQEINT32 * pCompCode, MQEINT32 * pReason);
extern  MQEENTRY 
MQEINT32  MQeFieldsGetArray( MQEHSESS hSess, MQEHFIELDS hFlds, MQECHAR * pName, MQEBYTE *pDataType, MQEINT32 sOff, 
                             MQEVOID * pDstBuf, MQEINT32 dstLen, MQEVOID * pBase, MQEINT32 * pCompCode , MQEINT32 * pReason );
extern  MQEENTRY 
MQEINT32  MQeFieldsGetByArrayOfFd( MQEHSESS hSess, MQEHFIELDS hFlds, MQEFIELD pFlds[], MQEINT32 nFlds, 
                                   MQEINT32 * pCompCode, MQEINT32 * pReason); 
extern  MQEENTRY 
MQEINT32  MQeFieldsGetByIndex ( MQEHSESS hSess, MQEHFIELDS hFlds, MQEINT32 startIndex, MQEFIELD pFds[], MQEINT32 nFlds, 
                                MQEINT32 * pCompCode, MQEINT32 * pReason); 
extern  MQEENTRY 
MQEINT32  MQeFieldsGetByStruct( MQEHSESS hSess, MQEHFIELDS hFlds, MQEVOID * pStruct,  MQEFIELDDESC pFsds[], 
                                MQEINT32 nFsds, MQEINT32 * pCompCode, MQEINT32 * pReason); 
extern  MQEENTRY 
MQEINT32  MQeFieldsHide  ( MQEHSESS hSess, MQEHFIELDS hFlds, MQECHAR * pName, MQEINT32, MQEINT32 * pCompCode, MQEINT32 * pReason);
extern  MQEENTRY 
MQEINT32  MQeFieldsPut   ( MQEHSESS hSess, MQEHFIELDS hFlds, MQECHAR * pName, MQEBYTE DataType, MQEVOID * pData, MQEINT32 nElements, 
                           MQEINT32 * pCompCode, MQEINT32 * pReason);
extern  MQEENTRY 
MQEVOID  MQeFieldsPutArray( MQEHSESS hSess, MQEHFIELDS hFlds, MQECHAR* pName, MQEBYTE datatype, MQEVOID * pData, MQEINT32 nElements, 
                            MQEINT32 * pCompCode, MQEINT32 * pReason); 
extern  MQEENTRY 
MQEINT32  MQeFieldsPutByArrayOfFd( MQEHSESS hSess, MQEHFIELDS hFlds, MQECONST MQEFIELD pFlds[], MQEINT32 nFlds,  MQEINT32 * pCompCode, MQEINT32 * pReason);
extern  MQEENTRY 
MQEINT32  MQeFieldsPutByStruct( MQEHSESS hSess, MQEHFIELDS hFlds, MQEVOID * pStruct,  MQEFIELDDESC pFsds[], MQEINT32 nFsds, 
                                MQEINT32 * pCompCode, MQEINT32 * pReason);
extern  MQEENTRY 
MQEINT32  MQeFieldsRead   ( MQEHSESS hSess, MQEHFIELDS hFlds, MQECHAR * pName, MQEBYTE DataType, MQEVOID * pDestBuf, 
                            MQEINT32 srcOff, MQEINT32 srcLen, 
                            MQEVOID *pBase, MQEINT32 * pCompCode, MQEINT32 * pReason);
extern  MQEENTRY 
MQEINT32  MQeFieldsRestore( MQEHSESS hSess, MQEHFIELDS hFlds, MQEINT32 dumpLen, MQEBYTE pData[], MQEINT32 dataLen, 
                            MQEINT32 * pCompCode, MQEINT32 * pReason); 
extern  MQEENTRY 
MQEINT32  MQeFieldsWrite  ( MQEHSESS hSess, MQEHFIELDS hFlds, MQECHAR * pName, MQEBYTE DataType, MQEINT32 dstOffset, MQEBYTE * pSrcBuf, MQEINT32 srcLen, 
                            MQEINT32 * pCompCode, MQEINT32 * pReason); 
extern  MQEENTRY 
MQEINT32  MQeFieldsType   ( MQEHSESS hSess, MQEHFIELDS hFlds, MQECHAR * pTypeName, MQEINT32 typeLen,
                            MQEINT32 * pCompCode, MQEINT32 * pReason); 

#if defined(__cplusplus)
}
#endif

#endif
